//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  Green
#property indicator_color2  LimeGreen
#property indicator_color3  Red
#property indicator_color4  Orange
#property indicator_color5  DarkGray
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  3

extern string TimeFrame        = "Current time frame";
extern int    OscPeriod       = 100;
extern int    OscSegments     = 8;
extern int    OscLevels       = 6;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;
extern bool   alertsPushNotif = false;

//
//
//
//
//

double histuu[];
double histud[];
double histdd[];
double histdu[];
double osc[];
double cross[];
double OscMultiplier;
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,histud); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,histuu); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,histdu); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,histdd); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,osc);
   SetIndexBuffer(5,cross);
      OscMultiplier = 1.0/OscSegments;
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" UCS oscillator ("+OscPeriod+","+OscSegments+","+OscLevels+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { histud[0] = limit+1; return(0); }
           if (timeFrame != Period())
           {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for(int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);               
                     osc[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",OscPeriod,OscSegments,OscLevels,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsPushNotif,4,y);
                     histuu[i] = EMPTY_VALUE;
                     histud[i] = EMPTY_VALUE;
                     histdd[i] = EMPTY_VALUE;
                     histdu[i] = EMPTY_VALUE;
                     while(true)
                     {
                        if (osc[i] >  OscMultiplier*OscLevels) { histuu[i] = osc[i]; break; }
                        if (osc[i] >                        0) { histud[i] = osc[i]; break; }
                        if (osc[i] < -OscMultiplier*OscLevels) { histdd[i] = osc[i]; break; }
                        if (osc[i] <                        0) { histdu[i] = osc[i]; break; }
                                                                               break;
                     }
               }
               return(0);
           }
  
   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      double hh = High[ArrayMaximum(High,OscPeriod,i)];
      double ll = Low [ArrayMinimum(Low ,OscPeriod,i)];
      double rn = hh-ll;
      double mu = rn*OscMultiplier;
      double ml = ll + mu*OscSegments/2;
         if (rn!=0)
               osc[i] = (Close[i]-ml)/(rn/2);
         else  osc[i] = 0;       
               histuu[i] = EMPTY_VALUE;
               histud[i] = EMPTY_VALUE;
               histdd[i] = EMPTY_VALUE;
               histdu[i] = EMPTY_VALUE;
               cross[i]  = 0;
         while(true)
         {
            if (osc[i] >  OscMultiplier*OscLevels) { histuu[i] = osc[i]; cross[i] =  2; break; }
            if (osc[i] >                        0) { histud[i] = osc[i]; cross[i] =  1; break; }
            if (osc[i] < -OscMultiplier*OscLevels) { histdd[i] = osc[i]; cross[i] = -2; break; }
            if (osc[i] <                        0) { histdu[i] = osc[i]; cross[i] = -1; break; }
                                                                         break;
         }
   }
   manageAlerts();
   return(0);
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (cross[whichBar] != cross[whichBar+1])
      {
         if (cross[whichBar] ==  2) doAlert(whichBar,"level "+DoubleToStr( OscMultiplier*OscLevels,2)+" crossed up");
         if (cross[whichBar] == -2) doAlert(whichBar,"level "+DoubleToStr(-OscMultiplier*OscLevels,2)+" crossed down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" "+timeFrameToString(_Period)+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" UCS oscillator "+doWhat;
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(Symbol()+" UCS oscillator",message);
          if (alertsPushNotif) SendNotification(message);
          if (alertsSound)     PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workLinr[][6];
double iLinr(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workLinr,0)!= Bars) ArrayResize(workLinr,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
   
      period = MathMax(period,1);
      workLinr[r][instanceNo] = price;
         double lwmw = period; double lwma = lwmw*price;
         double sma  = price;
         for(int k=1; k<period && (r-k)>=0; k++)
         {
            double weight = period-k;
                   lwmw  += weight;
                   lwma  += weight*workLinr[r-k][instanceNo];  
                   sma   +=        workLinr[r-k][instanceNo];
         }             
   
   return(3.0*lwma/lwmw-2.0*sma/period);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}